<?php

/**
 * Created by PhpStorm.
 * User: jderosa
 * Date: 20/04/20
 * Time: 10.45
 */

namespace FiloBlu\Rma\Plugin\Model;

use Exception;
use FiloBlu\Rma\Handlers\CancelChangeSizeOrderHandler;
use FiloBlu\Rma\Handlers\CreateChangeSizeOrderHandler;
use FiloBlu\Rma\Handlers\EasyReturnHandler;
use FiloBlu\Rma\Handlers\RmaEmailHandler;
use FiloBlu\Rma\Handlers\RmaItemStatusUpdateHandler;
use FiloBlu\Rma\Handlers\RmaPoliciesHandler;
use FiloBlu\Rma\Helper\RmaItemResolutionHistoryHelper;
use FiloBlu\Rma\Logger\Logger;
use Magento\Rma\Api\Data\RmaInterface;
use Magento\Rma\Model\Rma;
use Throwable;

/**
 *
 */
class RmaModelRmaProcessor
{
    /**
     * @var RmaPoliciesHandler
     */
    protected $rmaPoliciesHandler;
    /**
     * @var Logger
     */
    private $logger;
    /**
     * @var EasyReturnHandler
     */
    private $easyReturnHandler;
    /**
     * @var RmaItemStatusUpdateHandler
     */
    private $rmaItemStatusUpdateHandler;
    /**
     * @var RmaEmailHandler
     */
    private $rmaEmailHandler;
    /**
     * @var CancelChangeSizeOrderHandler
     */
    private $cancelChangeSizeOrderHandler;
    /**
     * @var CreateChangeSizeOrderHandler
     */
    private $createChangeSizeOrderHandler;
    /**
     * @var RmaItemResolutionHistoryHelper
     */
    private $resolutionHistoryHelper;

    /**
     * RmaModelRmaProcessor constructor.
     * @param Logger $logger
     * @param EasyReturnHandler $easyReturnHandler
     * @param RmaItemStatusUpdateHandler $rmaItemStatusUpdateHandler
     * @param RmaEmailHandler $rmaEmailHandler
     * @param CancelChangeSizeOrderHandler $cancelChangeSizeOrderHandler
     * @param CreateChangeSizeOrderHandler $createChangeSizeOrderHandler
     * @param RmaPoliciesHandler $rmaPoliciesHandler
     * @param RmaItemResolutionHistoryHelper $resolutionHistoryHelper
     */
    public function __construct(
        Logger $logger,
        EasyReturnHandler $easyReturnHandler,
        RmaItemStatusUpdateHandler $rmaItemStatusUpdateHandler,
        RmaEmailHandler $rmaEmailHandler,
        CancelChangeSizeOrderHandler $cancelChangeSizeOrderHandler,
        CreateChangeSizeOrderHandler $createChangeSizeOrderHandler,
        RmaPoliciesHandler $rmaPoliciesHandler,
        RmaItemResolutionHistoryHelper $resolutionHistoryHelper
    ) {
        $this->logger = $logger;
        $this->easyReturnHandler = $easyReturnHandler;
        $this->rmaItemStatusUpdateHandler = $rmaItemStatusUpdateHandler;
        $this->rmaEmailHandler = $rmaEmailHandler;
        $this->cancelChangeSizeOrderHandler = $cancelChangeSizeOrderHandler;
        $this->createChangeSizeOrderHandler = $createChangeSizeOrderHandler;
        $this->rmaPoliciesHandler = $rmaPoliciesHandler;
        $this->resolutionHistoryHelper = $resolutionHistoryHelper;
    }

    /**
     * @param Rma $subject
     * @param \Magento\Rma\Api\Data\RmaInterface|bool $result
     * @return Rma
     * @throws \Exception
     */
    public function afterSaveRma(RmaInterface $subject, $result)
    {
        if ($result === false) {
            return $subject;
        }

        try {

            //check if EasyRma is enabled then save EasyRma Fields
            $this->easyReturnHandler->saveEasyReturnFields($subject, $result);

            //Save Rma Item Status History
            $this->resolutionHistoryHelper->saveRmaItemsData($result);

            //generate changeSizeOrder if needed and send new changeSizeOrder email
            $this->createChangeSizeOrderHandler->checkIfCreateChangeSizeOrder($subject, $result);

            //check if update status to changeSizeOrder or auto approve rma based on BE configuration
            $this->rmaItemStatusUpdateHandler->shouldUpdateStatus($subject, $result);

            //send email if needed
            $this->rmaEmailHandler->sendRmaEmail($result);

            //check rma policies
            $this->rmaPoliciesHandler->checkPolicies($result);

        } catch (Exception $e) {
            $this->logger->critical($e->getMessage(), ['exception' => $e]);
            throw $e;
            //return $result;
        } catch (Throwable $t) {
            $this->logger->critical($t->getMessage(), ['exception' => $t]);
            throw $t;
            //return $result;
        }
        return $result;
    }

    /**
     * @param Rma $subject
     * @param $result
     * @return mixed
     */
    public function afterClose(Rma $subject, $result)
    {
        try {
            //cancel ChangeSizeOrder on rma cancel action
            $this->cancelChangeSizeOrderHandler->cancelChangeSizeOrder($subject, $result);
            //send email if needed
            $this->rmaEmailHandler->sendRmaEmail($result);
            return $result;
        } catch (Exception $e) {
            $this->logger->critical($e->getMessage(), ['exception' => $e]);
            return $result;
        } catch (Throwable $t) {
            $this->logger->critical($t->getMessage(), ['exception' => $t]);
            return $result;
        }
    }
}
